"""
MessageWindow.py
by Jason Conti
February 19, 2010

Popup window for viewing notifications.
"""

import glib
import gnomeapplet
import gobject
import gtk
import pango

from string import Template

message_template = Template("""<b>$SUMMARY</b>
$BODY
<small><i>$TIMESTAMP from <b>$APPNAME</b></i></small>""")

class MessageWindow(gtk.Window):
  """Shows the log of notification messages."""
  def __init__(self, parent):
    gtk.Window.__init__(self)

    self._parent = parent

    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
    self.set_decorated(False)
    self.set_resizable(False)
    self.stick()
    self.set_title("Message Window")
    self.set_border_width(0)
    self.set_size_request(400, 400)

    self._frame = gtk.Frame()
    self._frame.set_border_width(0)
    self.add(self._frame)

    self._vadjust = gtk.Adjustment()
    self._vadjust.connect("changed", self.on_vadjust_changed)

    self._scroll = gtk.ScrolledWindow(vadjustment=self._vadjust)
    self._scroll.set_border_width(5)
    self._scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
    self._frame.add(self._scroll)

    self._liststore = gtk.ListStore(gtk.gdk.Pixbuf, str)
    self._treeview = gtk.TreeView(self._liststore)
    self._treeview.set_headers_visible(False)
    self._treeview.set_rules_hint(True)
    self._treeview.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_HORIZONTAL)
    self._treeview.get_selection().set_select_function(lambda x: False)
    self._scroll.add(self._treeview)

    self._icon_column = gtk.TreeViewColumn()
    self._treeview.append_column(self._icon_column)

    self._icon_cell = gtk.CellRendererPixbuf()
    self._icon_cell.set_property("xalign", 0)
    self._icon_cell.set_property("yalign", 0)
    self._icon_cell.set_property("xpad", 5)
    self._icon_cell.set_property("ypad", 5)
    self._icon_column.pack_start(self._icon_cell, False)
    self._icon_column.add_attribute(self._icon_cell, "pixbuf", 0)

    self._message_column = gtk.TreeViewColumn()
    self._treeview.append_column(self._message_column)

    self._message_cell = gtk.CellRendererText()
    self._message_cell.set_property("wrap-width", 300)
    self._message_cell.set_property("wrap-mode", pango.WRAP_WORD_CHAR)
    self._message_cell.set_property("xpad", 5)
    self._message_cell.set_property("ypad", 5)
    self._message_column.pack_start(self._message_cell, True)
    self._message_column.add_attribute(self._message_cell, "markup", 1)

  def on_vadjust_changed(self, adjustment, *args):
    """Reset the adjustment position when the adjustment is changed (resized), but
    not moved, so the user can scroll around freely."""
    self._vadjust.set_value(0)

  def add_message(self, message):
    """Adds a message to the message window."""
    icon = message.get_icon(32)
    app_name = glib.markup_escape_text(message.app_name)
    timestamp = glib.markup_escape_text(message.formatted_timestamp())
    summary = glib.markup_escape_text(message.summary)
    body = glib.markup_escape_text(message.body)
    self._liststore.prepend([icon, message_template.substitute(SUMMARY=summary, 
      BODY=body, TIMESTAMP=timestamp, APPNAME=app_name)])

  def align_with_parent(self):
    """Adapted from position_calendar_popup in the GNOME clock applet."""
    # Get the size of the applet button and the size of this window
    x, y = self._parent.get_origin()
    w, h = self.get_size()
    button_w, button_h = self._parent.get_size()

    screen = self.get_screen()
    monitor = None

    # Figure out which monitor we are on
    for i in range(screen.get_n_monitors()):
      m = screen.get_monitor_geometry(i)

      if x >= m.x and x <= m.x + m.width and y >= m.y and y <= m.y + m.height:
        monitor = m
        break

    # Probably xinerama
    if monitor == None:
      monitor = gtk.gdk.Rectangle(0, 0, screen.get_width(), screen.get_height())

    orient = self._parent.get_orient()

    # Figure out the x, y coordinates of the window and the gravity depending
    # on the orientation of the panel
    if orient == gnomeapplet.ORIENT_RIGHT:  # Panel on left
      x += button_w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_WEST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_WEST

    elif orient == gnomeapplet.ORIENT_LEFT: # Panel on right
      x -= w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_EAST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_EAST

    elif orient == gnomeapplet.ORIENT_DOWN: # Panel on top
      y += button_h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_NORTH_WEST

    else: # Panel on bottom
      y -= h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_SOUTH_WEST

    self.move(x, y)
    self.set_gravity(gravity)

