"""
Notification.py
by Jason Conti
February 19, 2010

Monitors DBUS for org.freedesktop.Notifications.Notify messages, parses them,
and notifies listeners when they arrive.
"""

import dbus
import glib
import gobject
import gtk
import logging
import time

from dbus.mainloop.glib import DBusGMainLoop

from Icon import load_icon, load_icon_from_file

class ImageData(object):
  """Parses the image_data hint from a DBUS message."""
  def __init__(self, image_data):
    if len(image_data) < 7:
      raise Exception("Invalid image_data: " + repr(image_data))

    self.width = int(image_data[0])
    self.height = int(image_data[1])
    self.rowstride = int(image_data[2])
    self.has_alpha = bool(image_data[3])
    self.bits_per_sample = int(image_data[4])
    self.channels = int(image_data[5])
    self.data = self.dbus_array_to_str(image_data[6])

  def dbus_array_to_str(self, array):
    return "".join(map(chr, array))

  def get_pixbuf(self, size):
    """Creates a pixbuf from the image data and scale it to the appropriate size."""
    pixbuf = gtk.gdk.pixbuf_new_from_data(self.data, gtk.gdk.COLORSPACE_RGB,
        self.has_alpha, self.bits_per_sample, self.width, self.height,
        self.rowstride)

    return pixbuf.scale_simple(size, size, gtk.gdk.INTERP_BILINEAR)

class Message(gobject.GObject):
  """Parses a DBUS message in the Notify format specified at:
  http://www.galago-project.org/specs/notification/0.9/index.html"""
  # Message urgency
  LOW = 0
  NORMAL = 1
  CRITICAL = 2
  def __init__(self, dbus_message = None, timestamp = None):
    gobject.GObject.__init__(self)

    self.timestamp = timestamp

    args = dbus_message.get_args_list()

    if len(args) != 8:
      raise Exception("Invalid message args_list: " + repr(args))

    self.app_name = str(args[0])
    self.replaces_id = args[1]
    self.app_icon = args[2]
    self.summary = str(args[3])
    self.body = str(args[4])
    self.actions = args[5]
    self.hints = dict(args[6])
    self.expire_timeout = args[7]

    if "urgency" in self.hints:
      urgency = self.hints["urgency"]
      if urgency == 0:
        self.urgency = Message.LOW
      elif urgency == 2:
        self.urgency = Message.CRITICAL
      else:
        self.urgency = Message.NORMAL
    else:
      self.urgency = Message.NORMAL

    if "image_data" in self.hints:
      self.image_data = ImageData(self.hints["image_data"])
    else:
      self.image_data = None

    self.log_message()

  def get_icon(self, size = 48):
    """Loads the icon into a pixbuf. Adapted from the load_icon code in
    bubble.c of notify-osd."""
    icon_name = str(self.app_icon)
    
    # Try to load the pixbuf from a file
    if icon_name.startswith("file://") or icon_name.startswith("/"):
      icon = load_icon_from_file(icon_name, size)
      if icon != None:
        return icon

    # Try to load the pixbuf from the current icon theme
    elif icon_name != "":
      icon = load_icon(icon_name, size)
      if icon != None:
        return icon

    # Try to load the image data from the message
    elif self.image_data != None:
      return self.image_data.get_pixbuf(32)

    return self.get_default_icon(size)

  def get_default_icon(self, size = 48):
    """Attempts to load the default message icon, returns None on failure."""
    if self.urgency == Message.LOW:
      return load_icon("notification-low", size)
    elif self.urgency == Message.CRITICAL:
      return load_icon("notification-critical", size)
    else:
      return load_icon("notification-normal", size)

  def formatted_timestamp(self):
    """Returned the timestamp in a different format."""
    return time.strftime("%I:%M:%S %p", self.timestamp)

  def log_message(self):
    """Write debug info about a message."""
    logging.debug("-" * 50)
    logging.debug("Message created at: " + self.formatted_timestamp())
    logging.debug("app_name: " + repr(self.app_name))
    logging.debug("replaces_id: " + repr(self.replaces_id))
    logging.debug("app_icon: " + repr(self.app_icon))
    logging.debug("summary: " + repr(self.summary))
    logging.debug("body: " + repr(self.body))
    logging.debug("actions: " + repr(self.actions))
    logging.debug("hints: " + repr(self.hints))
    logging.debug("expire_timeout: " + repr(self.expire_timeout))
    logging.debug("-" * 50)

class Notification(gobject.GObject):
  """Monitors DBUS for org.freedesktop.Notifications.Notify messages, parses them,
  and notifies listeners when they arrive."""
  __gsignals__ = {
      "message-received": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [Message])
  }
  def __init__(self):
    gobject.GObject.__init__(self)

    self._match_string = "type='method_call',interface='org.freedesktop.Notifications',member='Notify'"

    DBusGMainLoop(set_as_default=True)
    self._bus = dbus.SessionBus()
    self._bus.add_match_string(self._match_string)
    self._bus.add_message_filter(self._message_filter)

  def close(self):
    self._bus.close()

  def _message_filter(self, connection, dbus_message):
    if dbus_message.get_member() == "Notify" and dbus_message.get_interface() == "org.freedesktop.Notifications":
      glib.idle_add(self.emit, "message-received", Message(dbus_message, time.localtime()))

def test_receiver(w, m):
  print "-"*50
  print "timestamp", m.formatted_timestamp()
  print "app_name", repr(m.app_name)
  print "replaces_id", repr(m.replaces_id)
  print "app_icon", repr(m.app_icon)
  print "summary", repr(m.summary)
  print "body", repr(m.body)
  print "actions", repr(m.actions)
  print "hints", repr(m.hints)
  print "expire_timeout", repr(m.expire_timeout)
  print "-"*50

def main():
  x = Notification()
  x.connect("message-received", test_receiver)
  gtk.main()

if gtk.pygtk_version < (2, 8, 0):
  gobject.type_register(Message)
  gobject.type_register(Notification)

if __name__ == '__main__':
  main()
